/* 

WSH/FFMPEG Transcoder
Copyright (c) 2009, DAMToolkit.com

Licence
Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
    * Neither the name of DAMToolkit.com nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Usage:

1. Change the settings in the SETTINGS section
2. Adjust the OPTIONS section to suit your requirements
3. Create a scheduled job running transcode.js
4. Deposit videos to be processed into VIDEO_IN_PATH

*/

// SETTINGS

// Change the paths to the required locations
var VIDEO_IN_PATH = "D:\\projects\\transcoder\\video-in\\";
var VIDEO_PROCESSING_PATH = "D:\\projects\\transcoder\\video-processing\\";
var VIDEO_SUCCEEDED_PATH = "D:\\projects\\transcoder\\video-succeeded\\";
var VIDEO_FAILED_PATH = "D:\\projects\\transcoder\\video-failed\\";
var VIDEO_PREVIEW_PATH = "D:\\projects\\transcoder\\video-preview\\";
var VIDEO_THUMBNAILS_PATH = "D:\\projects\\transcoder\\video-thumbnails\\";
var FFMPEG_PATH = "D:\\projects\\transcoder\\ffmpeg_bin\\ffmpeg.exe";
var RESULT_LOG_PATH = "D:\\projects\\transcoder\\results_log\\";

// The call back - will receive notifications of success/failure
var CALL_BACK_URL = "http://localhost/video_processed.ashx";

// FFMPEG video processing command options
var FFMPEG_PROCESSING_OPTIONS = "-f flv";

// How many seconds into the clip to generate a thumbnail
var THUMB_FRAME_OFFSET = 5

// The size of the thumbnail
var THUMB_FRAME_SIZE = "100x76"


// OPTIONS

// Set to false if you don't need http callbacks
var callback = true;

// Set this to false for less verbose output, this option will keep the command window open
var debug = true;

// Set this to false to prevent log files from being generated
var output_logging = true;


// This is used for sending failure messages to the http callback
function fail_job(error_string)
{
	WScript.echo("*** Error: "+error_string);
	if (callback){
		xmlhttp.open("GET", CALL_BACK_URL + "&success=-1&resultstr=" + error_string, false);
		xmlhttp.send();
		xmlhttp = 0;
	}
	objFSO = 0;
	oShell = 0;
	WScript.Quit();
}

function print(debug_string){
	WScript.Echo(debug_string);
	}


// Setup the required COMs
if (callback) {var xmlhttp = new ActiveXObject("Msxml2.XMLHTTP");}
var objFSO = new ActiveXObject("Scripting.FileSystemObject");

// Get the next file from VIDEO_IN_PATH (if there is one) and move it to the processing folder
f = objFSO.GetFolder(VIDEO_IN_PATH);
fc = new Enumerator(f.files);
if (!fc.atEnd()){ 
	source_file_path = fc.item();
	source_file_name = fc.item().name;
	objFSO.MoveFile(source_file_path,VIDEO_PROCESSING_PATH+source_file_name);
	source_file_path = VIDEO_PROCESSING_PATH+source_file_name	
}
else {
	if (debug){print("No files found");}
	WScript.Quit();
}

if (debug) { print("Source file: "+source_file_path); }

var output_file = "\""+VIDEO_PREVIEW_PATH+source_file_name+".flv\"";
var log_file = "\""+RESULT_LOG_PATH+source_file_name+".txt\"";

// Check FFMPEG.exe is still where we said it was
if (!objFSO.FileExists(FFMPEG_PATH)){fail_job("FFMPEG.exe not in stated location, tried this:"+FFMPEG_PATH);}

//  This sets the FFMPEG conversion command spec
var ffmpeg_options = " -i \""+source_file_path+"\" "+FFMPEG_PROCESSING_OPTIONS+" -y "+output_file;

// If logging is required (true by default) add this option to the options
if (output_logging) {ffmpeg_options+=" 2> "+log_file;}


ffmpeg_command = FFMPEG_PATH + ffmpeg_options;
if (debug){print("\n\rFFMPEG command spec:\r\n"+ffmpeg_command);}

// Change to /K to keep the FFMPEG command shell open and see if there are errors, use /C to close the shell after processing
var oShell = WScript.createObject("WScript.Shell"); 
oShell.Run(("%comspec% /C "+ffmpeg_command),1,true); 

// Find out if the file was created, if not, assume a fail condition
if (!objFSO.FileExists(VIDEO_PREVIEW_PATH+source_file_name+".flv")){

	// Move the failed file to another folder to stop it getting processed again
	var failed_file_path = VIDEO_FAILED_PATH+source_file_name;
	if (debug){print("\n\rMoving source: "+source_file_path+" to failed folder:"+failed_file_path);}
	objFSO.MoveFile(source_file_path , failed_file_path)
	fail_job("Conversion failed, check log files for full details. Output file does not exist"+output_file);
}

//
// This section deals with the thumbnail
//

// Set up the path for the thumbnail and set the options for generating it
var thumbnail_output_file = "\""+VIDEO_THUMBNAILS_PATH+source_file_name+".jpg\"";
var ffmpeg_thumb_options = " -i \""+source_file_path+"\" -f image2 -ss "+THUMB_FRAME_OFFSET+" -vframes 1 -s "+THUMB_FRAME_SIZE+" "+thumbnail_output_file;
var thumb_log_file = "\""+RESULT_LOG_PATH+source_file_name+"_thumb.txt\"";

// Add the logging flag if required
if (output_logging) {ffmpeg_thumb_options+=" 2> "+thumb_log_file;}

// Generate the thumbnail with FFMPEG
var thumb_ffmpeg_command = FFMPEG_PATH + ffmpeg_thumb_options;
if (debug){print("\n\rFFMPEG thumbnail command spec:\r\n"+thumb_ffmpeg_command);}
oShell.Run(("%comspec% /C "+thumb_ffmpeg_command),1,true); 

// Find out if the thumbnail was created and fail the job if not 
if (!objFSO.FileExists(VIDEO_THUMBNAILS_PATH+source_file_name+".jpg")){
	fail_job("Thumbnail creation failed, check log files for full details. Output file does not exist");
} 
else {

	// Move the source file to the succeeded folder so it does not get re-processed
	var transcoded_file_path = VIDEO_SUCCEEDED_PATH+source_file_name;
	if (debug){print("\n\rMoving source: "+source_file_path+" to succeeded folder:"+transcoded_file_path);}
	objFSO.MoveFile(source_file_path , transcoded_file_path)
	
	// Send a message back to the callback script via http to say all is OK
	if (callback){
		xmlhttp.open('GET',CALL_BACK_URL+'?success=1&resultstr=OK',false);
		xmlhttp.send();
	}
	if (debug){WScript.echo("*** Success: conversion completed");}
	objFSO = 0;
	xmlhttp=0;
}